/*
 * GNU.FREE 2001
 *
 * Copyright (c) 1999, 2000, 2001 The Free Software Foundation (www.fsf.org)
 *
 * GNU.FREE Co-ordinator: Jason Kitcat <jeep@free-project.org>
 *
 * GNU site: http://www.gnu.org/software/free/
 * 
 * FREE e-democracy site: http://www.free-project.org
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program (COPYING); if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
  
package Free;

import java.util.*;
import Free.util.*;

/**
 * Processes the Strings passed to it by <code>TCPServer</code> but throws
 * all errors back up to <code>TCPServer</code> which can better handle
 * problems by elegantly closing connections.
 *
 * Also included are data structures and methods for storing and retrieving 
 * check date/time stamps for the FREE security system to provide persistence
 * between calls of <code>process()</code>.
 * 
 * @version 1.5.5  11 April 2001
 * @author Jason Kitcat
 */
public class ServerProtocol {

	private static Vector checkDates = new Vector(1000);
	private static Vector checkVotes = new Vector(1000);
	private static Vector checkAuthKey = new Vector(1000);
	
	/*
	 * Everything above is to keep the protocols neatly in this class
	 * and thus maintain persistence between calls of process()
	 * I feel this undoubtedly makes for clearer reading overall.
	 */
	 

	
	/**
	 * adds an entry to the checkVotes Vector for later use.
	 *
	 * @param  n  The index at which to insert
	 * @param  data  Packet message string to store
	 */
	 private static void setVotes(int n, String data) {
	 
	 	checkVotes.insertElementAt(data, n);
	 
	 } //EOF setVotes


	/**
	 * adds an entry to the checkDate Vector for later use.
	 *
	 * @param  n  The index at which to insert
	 * @param  data  Packet message string to store
	 */
	 private static void setCheck(int n, String data) {
	 
	 	checkDates.insertElementAt(data, n);
	 
	 } //EOF setCheck


	/**
	 * adds an entry to the checkAuthKey Vector for later verifications.
	 *
	 * @param cdate   Date to insert
	 * @returns    The index of where the date was inserted
	 */
	 private static int setKey(String akey) {
	 		
	 	int r = checkAuthKey.size();

	 	checkAuthKey.insertElementAt(akey, r);
	 		 	
	 	return r;
	 
	 } //EOF setKey
	 
	 
	 /**
	  * tries to find a matching key for security check.
	  *
	  * @param cdate   Key to find & match
	  *
	  * @returns   A string array with the result of the matching vote and the associated AuthKey
	  */
	  private static String[] getKey(String akey) {
	  
	  	int i=0;
	  	boolean test=false;
	  	String[] r = new String[] {"","",""};
	  	
		i = checkAuthKey.indexOf(akey);

		if (i==-1) {
			r[0] = "FALSE";
		} else {
			r[0] = checkVotes.elementAt(i).toString(); // collect vote
			r[1] = checkAuthKey.elementAt(i).toString(); // collect AuthKey
			r[2] = checkDates.elementAt(i).toString(); // collect Date
			checkDates.removeElementAt(i);  // clear data
 			checkAuthKey.removeElementAt(i);
 			checkVotes.removeElementAt(i);
 		}

	  	return r;
	  		
	} //EOF getKey
	  
	  
	/**
	 * <code>process</code> analyses Strings sent from <code>TCPServer</code>
	 * and decides whether the data is in fact valid packets.
	 *
	 * If so then the appropriate replies are formulated and returned to 
	 * <code>TCPServer</code> for sending to the client.
	 *
	 * @param inputData   Contains the contents of a received packet
	 * @returns   String with a packet to return to client
	 */
	protected static String process(String inputData) throws Exception {

		String outputData="";
		
		if (inputData.charAt(0) == 'C') {
		
			/* vote or total confirm */
			Packet p = new Packet(inputData);
			
			if (AuthSys.checkDigest(p)) {  // tamper check
			
				// parse message
				String[] pp = p.splitMessage();
				String FCAuthKey = pp[0];
				String check = pp[1];
				
				String[] t = getKey(FCAuthKey); //compare second time stamp
			
				/* respond according to regional or totaller mode */
				if (RTServer.serverType == 'R') {
					if (!t[0].equals("FALSE")&&check.equals(t[2])) {  //date security ok (FALSE indicated no matches, second term checks if dates match)
							/* register vote */														
							try {
								// AuthKey check, if ok store vote
								int kcResult = DBase.authKeyCheck(t[1]);
								if (kcResult==1) {
									/* Normal vote */
									// now store the vote							
									DBase.storeVote(t[0]);
									RTServer.NORM.info("Vote stored in database");
									// confirm vote to user
									outputData = "C|OK|" + AuthSys.makeDigest("OK") + "\r\n";
								} else if (kcResult==2) {
									/* Test Ballot */
									RTServer.NORM.info("TEST BALLOT RECEIVED - Value: " + t[0]);
									// confirm vote to user
									outputData = "C|OK|" + AuthSys.makeDigest("OK") + "\r\n";
								} else {
									/* we have an error */
									RTServer.NORM.error("AuthKey Check failed!");
									outputData = "ERROR";
								}
								
							} catch (Exception e) {
								RTServer.NORM.error("Vote registration error: " + e.getMessage());
								outputData = "ERROR";
							}
					} else {
							RTServer.NORM.warn("Vote security failure");
							outputData = "ERROR";
					}
				} else {
					if (!t[0].equals("FALSE")&&check.equals(t[2])) {  //date security ok
							/* register total */
							try {
								Packet px = new Packet("X|"+t[0]+"|XXX"); // split info
								String[] ppx = px.splitMessage();
								DBase.storeTotal(ppx[0], ppx[1]); // store
								RTServer.NORM.info("Total stored in database");
								outputData = "C|OK|" + AuthSys.makeDigest("OK") + "\r\n"; //confirm total
							} catch (Exception e) {
								RTServer.NORM.error("Totals registration error: " + e.getMessage());
								outputData = "ERROR";
							}
					} else {
							RTServer.NORM.warn("Totals security failure.");
							outputData = "ERROR";
					}
				}
				
			} else {
				RTServer.NORM.warn("Packet corrupted or altered!");
				outputData = "ERROR";
			}
			
		} else if (inputData.charAt(0) == 'D') {
		
			/* diagnostic */
			Packet p = new Packet(inputData);
			
			if (AuthSys.checkDigest(p)) {  // tamper check
			
				RTServer.NORM.info("Diagnostic packet received");
			
				if (p.getMessage().equals("TEST")) {
					outputData= "D|OK|" + AuthSys.makeDigest("OK") + "\r\n";
				} else {
					RTServer.NORM.warn("Unrecognized diagnostic command.");
					outputData = "ERROR";
				}
			
			} else {
				RTServer.NORM.warn("Packet corrupted or altered!");
				outputData = "ERROR";
			}
			
		} else if (inputData.charAt(0) == 'T') {
		
			/* total for party */
			Packet p = new Packet(inputData);
			
			if (AuthSys.checkDigest(p)) {  // tamper check
			
				if (p.getMessage().equals("DONE")) {
					RTServer.rServersDone = RTServer.rServersDone + 1;
					RTServer.NORM.info("Received " + RTServer.rServersDone + "/" + RTServer.rServers + " totals.");
					if (RTServer.rServersDone == RTServer.rServers) { // if all results in
						DBase.calcResult();
					}
					outputData = "C|DONE|" + AuthSys.makeDigest("DONE") + "\r\n"; // confirm
				} else {
					// commence security check
					
					// split packet message
					String[] pp = p.splitMessage();
					String tkey = pp[0];
					String check = pp[1];
					String total = pp[2] + "-" + pp[3];

					if (total.equals("")|total.equals(" ")) {  // check to see if there is a total in the data
						RTServer.NORM.warn("No total data in packet");
						outputData = "ERROR";
					} else {
						// commence security check						
						//store time stamp, and total packet
						int checkIndex = setKey(tkey);
						setVotes(checkIndex, total);
						setCheck(checkIndex, check);
						outputData = "C|STAMP|" + AuthSys.makeDigest("STAMP") + "\r\n"; //ask for time stamp check
					}
				}
			} else {
				RTServer.NORM.warn("Packet corrupted or altered!");
				outputData = "ERROR";
			}

		} else if (inputData.charAt(0) == 'V') {
		
			/* normal vote */
			if (inputData.length() > 120) {  // simple validity check to save work
			
				Packet p = new Packet(inputData);
			
				if (AuthSys.checkDigest(p)) {  // tamper check

					// split packet message into vote, AuthKey and time stamp					
					String[] pp = p.splitMessage();
					String check = pp[0];
					String vote = pp[1];
					String FCAuthKey = pp[2];
						
					if (vote.equals("")|vote.equals(" ")) {  // check to see if there is a party in the data
						RTServer.NORM.warn("No vote data in packet");
						outputData = "ERROR";
					} else {
						// commence security check						
						//store time stamp, AuthKey and vote packet
						int checkIndex = setKey(FCAuthKey);
						setVotes(checkIndex, vote);
						setCheck(checkIndex, check);
						outputData = "C|STAMP|" + AuthSys.makeDigest("STAMP") + "\r\n"; //ask for time stamp check
					}
				} else {
					RTServer.NORM.warn("Packet corrupted or altered!");
					outputData = "ERROR";
				}
			} else {
				RTServer.NORM.warn("Bad packet!");
				outputData = "ERROR";
			}
			
		} else if (inputData.charAt(0) == 'X') {
		
			/* end communication*/
			Packet p = new Packet(inputData);
			
			if (AuthSys.checkDigest(p)) {  // tamper check
			
				outputData = "DONE";
			} else {
				RTServer.NORM.warn("Packet corrupted or altered!");
				outputData = "ERROR";
			}

		} else {
				RTServer.NORM.warn("Unrecognized packet received.");
				outputData = "ERROR";
		}
		
		return outputData;

	}  //EOF process


} //EOF Class
